//+------------------------------------------------------------------+
//|                                                       Tandem.mqh |
//|                                 Copyright 2012, Evgeniy Trofimov |
//|                        https://login.mql5.com/ru/users/EvgeTrofi |
//+------------------------------------------------------------------+
//|                 |
//|    .                                   |
//|         :                                          |
//|                                                                  |
//|   http://www.mql5.com/ru/code/955 -             |
//|                                                                  |
//|                                                                  |
//|                                           |
//|   #include <EvgeTrofi\Tandem.mqh>                                |
//+------------------------------------------------------------------+
#include <EvgeTrofi\MoneyManagment.mqh>
#include <Trade\Trade.mqh>
enum ENUM_TANDEM_LOT{    //     
   TANDEM_LOT_NONE,      //      .
   TANDEM_LOT_LEONID553, //   ,   "  " (http://articles.mql4.com/ru/403)
   TANDEM_LOT_EDIK1975   //    ,   "" (http://codebase.mql4.com/ru/8573)
};
struct FUTURE_TRANSACTION{  //   ,     
   string Sym;              //   
   int Type;                //  : 0 - , 1 - 
   double Lot;              //   
};
//+------------------------------------------------------------------+
class CTandem{
private:
public:
//   
                CTandem(void);
               ~CTandem(void){};
//      :
      string    Symbol1; //  
      string    Symbol2; //  
      double    Volume1; //     
      double    Volume2; //     
      double    Lot1;    //  
      double    Lot2;    //  
ENUM_TANDEM_LOT MethodVolume; //,    Volume1  Volume2
//         
      double    CalcVolumes(int hATR1, int hATR2);
      double    CalcVolumes(int inPeriod);
      void      CalcLots(double inLoadPercent);
      void      BasketOpen(FUTURE_TRANSACTION &FT[], ulong inMagic=0, int inCorrect=0);
      void      BasketClose(FUTURE_TRANSACTION &FT[], ulong inMagic=0);
};
//+------------------------------------------------------------------+
CTandem::CTandem(void){
   Symbol1 = "";
   Symbol2 = "";
   Volume1 = 0;
   Volume2 = 0;
   Lot1 = 0;
   Lot2 = 0;
   MethodVolume = TANDEM_LOT_NONE;
}
//+------------------------------------------------------------------+
//       
double CTandem::CalcVolumes(int hATR1, int hATR2){
   //hATR -   ATR,        ,   
   MethodVolume = TANDEM_LOT_NONE;
   double ATR1[], ATR2[];
   if(CopyBuffer(hATR1, 0, 1, 1, ATR1)<=0){
      Print("    ATR   "+Symbol1);
      return(0);
   }
   if(IsStopped()) return(0); //Checking for stop flag
   if(CopyBuffer(hATR2, 0, 1, 1, ATR2)<=0){
      Print("    ATR   "+Symbol2);
      return(0);
   }
   //     
   CMoneyManagment MM;
   double kVol1=MM.TickValue(Symbol1)/SymbolInfoDouble(Symbol1,SYMBOL_TRADE_TICK_SIZE);
   double kVol2=MM.TickValue(Symbol2)/SymbolInfoDouble(Symbol2,SYMBOL_TRADE_TICK_SIZE);
   Volume1 = 1;
   Volume2 = Volume1 * kVol1 * ATR1[0] / kVol2 / ATR2[0];
   MethodVolume = TANDEM_LOT_LEONID553;
   return(Volume2);
}//CalcVolumes()
//+------------------------------------------------------------------+
//       
double CTandem::CalcVolumes(int inPeriod){
   MethodVolume = TANDEM_LOT_NONE;
   MqlRates Price1[], Price2[];
   if(CopyRates(Symbol1, Period(), 1, inPeriod, Price1)<=0){
      Print("  "+Symbol1+" !");
      return(0);
   }
   if(CopyRates(Symbol2, Period(), 1, inPeriod, Price2)<=0){
      Print("  "+Symbol2+" !");
      return(0);
   }
   double sum_p1=0, sum_p2=0;
   for(int i=0; i<inPeriod; i++){
      sum_p1=sum_p1+(Price1[i].high-Price1[i].low);
      sum_p2=sum_p2+(Price2[i].high-Price2[i].low);
   }
   sum_p1=sum_p1/inPeriod;
   sum_p2=sum_p2/inPeriod;
   Volume1=1;
   CMoneyManagment MM;
   double kVol1=MM.TickValue(Symbol1)/SymbolInfoDouble(Symbol1,SYMBOL_TRADE_TICK_SIZE);
   double kVol2=MM.TickValue(Symbol2)/SymbolInfoDouble(Symbol2,SYMBOL_TRADE_TICK_SIZE);
   Volume2=(sum_p1*kVol1)/(sum_p2*kVol2);
   MethodVolume = TANDEM_LOT_EDIK1975;
   return(Volume2);
}//CalcVolumes()
//+------------------------------------------------------------------+
//       ,    
// inLoadPercent,    
//  Volume1  Volume2.
void CTandem::CalcLots(double inLoadPercent){
   if(MethodVolume == TANDEM_LOT_NONE) {
      Print("  CalcLots()        CalcVolumes()");
      return;
   }
   CMoneyManagment MM;
   if(inLoadPercent>0){
      double NeedMargin = AccountInfoDouble(ACCOUNT_EQUITY) * inLoadPercent / 100;
      double k = NeedMargin / (MM.GetMargin(Symbol1)*Volume1 + MM.GetMargin(Symbol2)*Volume2);
      Lot1 = MM.LotNormalize(k * Volume1, Symbol1);
      Lot2 = MM.LotNormalize(k * Volume2, Symbol2);
   }else{
      Lot1 = MM.LotNormalize(Volume1);
      Lot2 = MM.LotNormalize(Volume2);
   }
}//CalcLots()
//+------------------------------------------------------------------+
//       FT

void CTandem::BasketOpen(FUTURE_TRANSACTION &FT[], ulong inMagic=0, int inCorrect=0){
   double CurrentLot=0;
   CMoneyManagment MM;
   CTrade Trade;
   Trade.SetExpertMagicNumber(MagicNumber);
   for(int i = 0; i < ArraySize(FT); i++){
      //,      
      CurrentLot = MM.GetCurrentLot(FT[i].Sym, inMagic);
      if(inCorrect==0) {
         if(Znak(CurrentLot)==Znak(FT[i].Type)) continue;
         FT[i].Lot=FT[i].Lot+MathAbs(CurrentLot);
      }else{
         if(CurrentLot==FT[i].Lot) continue; 
         if(Znak(CurrentLot)==Znak(FT[i].Type)) {
            if(MathAbs(CurrentLot)>FT[i].Lot){
               FT[i].Type = Reverce(FT[i].Type);
               FT[i].Lot = MathAbs(CurrentLot)-FT[i].Lot;
            }else{
               FT[i].Lot = FT[i].Lot-MathAbs(CurrentLot);
            }
            //    ,     !
            if(MM.LotIsKiller(FT[i].Lot, FT[i].Sym, FT[i].Type)) continue;
         }else{
            FT[i].Lot=FT[i].Lot+MathAbs(CurrentLot);
         }
      }
      if(MM.LotIsKiller(FT[i].Lot, FT[i].Sym, FT[i].Type)) FT[i].Lot=FT[i].Lot+SymbolInfoDouble(FT[i].Sym, SYMBOL_VOLUME_STEP);
      if(FT[i].Type==0){
         Trade.Buy(FT[i].Lot, FT[i].Sym);
      }else{
         Trade.Sell(FT[i].Lot, FT[i].Sym);
      }
      if(Trade.ResultRetcode()>0) Print(Trade.ResultRetcodeDescription());
   }//Next i
}//BasketOpen()
//+------------------------------------------------------------------+
void CTandem::BasketClose(FUTURE_TRANSACTION &FT[],ulong inMagic=0){
   CTrade Trade;
   Trade.SetExpertMagicNumber(MagicNumber);
   for(int i=0; i<ArraySize(FT); i++){
      if(FT[i].Type==0){
         Trade.Buy(FT[i].Lot, FT[i].Sym);
      }else{
         Trade.Sell(FT[i].Lot, FT[i].Sym);
      }
      if(Trade.ResultRetcode()>0) Print(Trade.ResultRetcodeDescription());
   }   
}//BasketClose()
//+------------------------------------------------------------------+
int Znak(double Value){
   if(Value>0) return(1);
   if(Value<0) return(-1);
   return(0);
}
int Znak(int Value){
   if(Value==0) return(1);
   return(-1);
}
//+------------------------------------------------------------------+
int Reverce(int inType){
   if(inType==0) return(1);
   return(0);
}
//+------------------------------------------------------------------+
